#ifndef MATRIX_H
#define MATRIX_H

// File: matrix.h
// --------------
// Definition of template class Matrix and functions.
// The Matrix class is a template for doing matrix-calculating
// on abstract objects.
//
// (c) 1998-2000 MeeSoft - Rune M. Barnkob
//
// YOU MAY FREELY DISTRIBUTE AND USE THIS FILE - JUST CREDIT ORIGINAL AUTHOR.
//
// Contents:
// ---------
// Matrix
//  constructors:
//             () - creates 1x1 matrix
//             ( int rows, int columns ) - creates rows x columns
//  operators: +(matrix&)
//             -(matrix&), -()
//             *(matrix&, matrix&), *(matrix&,TYPE), *(TYPE,matrix&)
//             !() - returns transposed
// %           ~() - inverse matrix (not always possible!)
//  functions:
//             .at( row, col ) - return element at...
//             .print(ostream&) - prints to std.out.
//             .rows() - return number of rows
//             .cols() - return number of columns
//%            .singular() returns boolean on singular
//%            .det() - returns determinant
//%            .Gauss() - performs complete gauss on matrix
//%            .Gauss( int column ) - performs gauss until column
//%            .rho() - returns rho

#include <iostream>
#include <iomanip>
#include <assert.h>
//#define DEBUG(s) (cerr<<s<<endl)
#define DEBUG(_)

template <class TYPE = double>
class Matrix {
  private:
    // array with fields
    TYPE *data;
    // dimensions of matrix
    int fRows, fColumns;

  public:
    // Normal constructor (rows, cols) (default is 1x1)
    Matrix(int=1, int=1);
		
    // Copy constructor ( copies all fields (deep copy) )
    Matrix(const Matrix<TYPE>&);

    // Destructor
    ~Matrix();

    // support for io-streams (use print(ostream&) for formattet multiline output)
    ostream& print(ostream& o);

    // Returns number of rows
    int rows() { return fRows; }
		
    // Returns number of columns
    int cols() { return fColumns; }
		
		
    // Returns element at:
    inline TYPE& at(int row, int col) const;

    // Assignment operator (for deep copying)
    Matrix<TYPE>& operator =(const Matrix<TYPE>& b);
		
    // Addition of 2 matrices
    Matrix<TYPE> operator +(const Matrix<TYPE>& b) const;

    // Subtraction of 2 matrices
    Matrix<TYPE> operator -(const Matrix<TYPE>& b) const;
    // Unary -
    Matrix<TYPE> operator -() const;

    
    // Multiplication of 2 matrices, matrix with const and const with matrix
    friend Matrix<TYPE> operator *<TYPE>(const Matrix<TYPE>&, const Matrix<TYPE>&);
    friend Matrix<TYPE> operator *<TYPE>(TYPE, const Matrix<TYPE>&);
    friend Matrix<TYPE> operator *<TYPE>(const Matrix<TYPE>&, TYPE);
    /**/

    // Transpose Matrix
    Matrix<TYPE> operator!() const;

    /*
    // Determin inverse matrix (may fail!)
    Matrix<TYPE> operator~() const;
    
    // Join matrices at length eg: [[1,1],[1,1]]<<[[2,2,2],[2,2,2]] = [[1,1,2,2,2],[1,1,2,2,2]];
    Matrix<TYPE> operator <<(Matrix<TYPE>& b)
  		
    // RowSwap - swap two rows
    void RowSwap(int Row1, int Row2)
  
    // ColSwap - swap two columns
    void ColSwap(int Col1, int Col2)
  
    // RowAdd - add a row to another row
    void RowAdd(int DestRow, extended Factor, int SourceRow)
  
    // RowMul - Multiply Row with Factor
    void RowMul(int Row, extended Factor)
    */

}; // Matrix<TYPE>

// General constructor
template <class TYPE>
Matrix<TYPE>::Matrix(int rows=1, int cols=1) :
  fRows(rows), fColumns(cols) {
  DEBUG("Matrix<TYPE>::Matrix(_,_) - normal");
  data = new TYPE[fRows*fColumns];
}

// Copy constructor
template <class TYPE>
Matrix<TYPE>::Matrix(const Matrix<TYPE>& c) :
  fRows(c.fRows), fColumns(c.fColumns) {
  DEBUG("Matrix<TYPE>::Matrix(_) - copy version");
  data = new TYPE[fRows*fColumns];
  int TYPEs = fColumns*fRows;
  for (int cc=0; cc<TYPEs; cc++)
    data[cc] = c.data[cc];
}

// Destructor (deallocate storage)
template <class TYPE>
Matrix<TYPE>::~Matrix() {
  delete[] data;
}

// Print (formatted) the matrix on a stream
template <class TYPE>
ostream& Matrix<TYPE>::print(ostream& o) {
  DEBUG("Matrix<TYPE>::print(_)");
  for (int rc=0; rc<fRows; rc++) {
    o << "";
    for (int cc=0; cc<fColumns; cc++)
      o << setw(8) << at(rc,cc);
    o << "" << endl;
  }
  return o;
}

// index array on address
template <class TYPE>
inline TYPE& Matrix<TYPE>::at(int row, int col) const {
//  DEBUG("Matrix<TYPE>::at("<<row<<","<<col<<")");
  assert( data!=NULL );
  assert( row<fRows && col<fColumns );

  return data[row*fColumns+col];
}

// Assignment operator (for deep copying)
template <class TYPE>
inline Matrix<TYPE>& Matrix<TYPE>::operator=(const Matrix<TYPE>& b) {
  DEBUG("Matrix<TYPE>::operator=(_)");
  assert( fRows==b.fRows && fColumns==b.fColumns );

  int TYPEs = fColumns*fRows;
  for (int cc=0; cc<TYPEs; cc++)
    data[cc] = b.data[cc];

  return *this;
}


// Addition of 2 matrices
template <class TYPE>
Matrix<TYPE> Matrix<TYPE>::operator+(const Matrix<TYPE>& b) const {
  DEBUG("Matrix<TYPE>::operator+(_)");

  assert( fRows==b.fRows && fColumns==b.fColumns );
  DEBUG("  - assertion ok");

  Matrix<TYPE> t(fRows, fColumns);
  DEBUG("  - construction ok");

  for (int rc=0; rc<t.fRows; rc++)
    for (int cc=0; cc<t.fColumns; cc++)
      t.at(rc,cc) = at(rc,cc) + b.at(rc,cc);
  DEBUG("  - done adding");

  return t;
}

// Subtraction of 2 matrices
template <class TYPE>
Matrix<TYPE> Matrix<TYPE>::operator-(const Matrix<TYPE>& b) const {
  DEBUG("Matrix<TYPE>::operator-(_)");
  assert( fRows==b.fRows && fColumns==b.fColumns );

  Matrix<TYPE> t(fRows, fColumns);

  for (int rc=0; rc<fRows; rc++)
    for (int cc=0; cc<fColumns; cc++)
      t.at(rc,cc) = at(rc,cc) - b.at(rc,cc);

  return t;
}

// Unary -
template <class TYPE>
Matrix<TYPE> Matrix<TYPE>::operator -() const {
  DEBUG("Matrix<TYPE>::operator-()");
  Matrix<TYPE> t(fRows, fColumns);

  for (int rc=0; rc<fRows; rc++)
    for (int cc=0; cc<fColumns; cc++)
      t.at(rc,cc) = - at(rc,cc);

  return t;
}




// dimensions must match when multiplying a matrix by another matrix
template <class TYPE>
Matrix<TYPE> operator *(const Matrix<TYPE>& a, const Matrix<TYPE>& b) {
  DEBUG("Matrix::* - a="<<a.fRows<<"x"<<a.fColumns<<"  b="<<b.fRows<<"x"<<b.fColumns);
  assert( a.fColumns == b.fRows );
  Matrix<TYPE> r(a.fRows, b.fColumns);
  for (int rc=0; rc<r.fRows; rc++)
    for (int cc=0; cc<r.fColumns; cc++) {
      TYPE prod;
      for (int m=0; m<a.fColumns; m++)
        prod += a.at(rc, m) * b.at(m, cc);
      r.at(rc,cc) = prod;
    }
  return r;
}

// Multiply by constant
template <class TYPE>
Matrix<TYPE> operator *(TYPE a, const Matrix<TYPE>& b) {
  Matrix<TYPE> r(b.fRows, b.fColumns);
  for (int rc=0; rc<r.fRows; rc++)
    for (int cc=0; cc<r.fColumns; cc++)
      r.at(rc,cc) = a*b.at(rc,cc);
  return r;
}
template <class TYPE>
Matrix<TYPE> operator *(const Matrix<TYPE>& b, TYPE a) {
  Matrix<TYPE> r(b.fRows, b.fColumns);
  for (int rc=0; rc<r.fRows; rc++)
    for (int cc=0; cc<r.fColumns; cc++)
      r.at(rc,cc) = a*b.at(rc,cc);
  return r;
}
/**/


// Transpose Matrix
template <class TYPE>
Matrix<TYPE> Matrix<TYPE>::operator!() const {
  DEBUG("Matrix<TYPE>::operator!()");
  Matrix t(fColumns, fRows);

  for (int rc=0; rc<fRows; rc++)
    for (int cc=0; cc<fColumns; cc++)
      t.at(cc,rc) = at(rc,cc);

  return t;
}

#endif
